@@ -8,31 +8,30 @@
  ------------- Copyright (C) 2001  Jon S. Berndt (jon@jsbsim.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU Lesser General Public License as published by the Free Software
- Foundation; either version 2 of the License, or (at your option) any later
- version.
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option) any
+ later version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU Lesser General Public License along with
- this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Lesser General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 59
+ Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
- Further information about the GNU Lesser General Public License can also be found on
- the world wide web at http://www.gnu.org.
+ Further information about the GNU Lesser General Public License can also be
+ found on the world wide web at http://www.gnu.org.
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
+#include <sstream>  // for assembling the error messages / what of exceptions.
+#include <stdexcept>  // using domain_error, invalid_argument, and length_error.
 #include "FGXMLElement.h"
-
-#include <cmath>
-#include <cstdlib>
-#include <iostream>
+#include "FGJSBBase.h"
 
 using namespace std;
 
@@ -42,9 +41,6 @@ FORWARD DECLARATIONS
 
 namespace JSBSim {
 
-static const char *IdSrc = "$Id: FGXMLElement.cpp,v 1.32 2011/02/13 00:42:45 jberndt Exp $";
-static const char *IdHdr = ID_XMLELEMENT;
-
 bool Element::converterIsInitialized = false;
 map <string, map <string, double> > Element::convert;
 
@@ -57,6 +53,7 @@ Element::Element(const string& nm)
   name   = nm;
   parent = 0L;
   element_index = 0;
+  line_number = -1;
 
   if (!converterIsInitialized) {
     converterIsInitialized = true;
@@ -64,6 +61,8 @@ Element::Element(const string& nm)
     // Length
     convert["M"]["FT"] = 3.2808399;
     convert["FT"]["M"] = 1.0/convert["M"]["FT"];
+    convert["CM"]["FT"] = 0.032808399;
+    convert["FT"]["CM"] = 1.0/convert["CM"]["FT"];
     convert["KM"]["FT"] = 3280.8399;
     convert["FT"]["KM"] = 1.0/convert["KM"]["FT"];
     convert["FT"]["IN"] = 12.0;
@@ -73,6 +72,8 @@ Element::Element(const string& nm)
     // Area
     convert["M2"]["FT2"] = convert["M"]["FT"]*convert["M"]["FT"];
     convert["FT2"]["M2"] = 1.0/convert["M2"]["FT2"];
+    convert["CM2"]["FT2"] = convert["CM"]["FT"]*convert["CM"]["FT"];
+    convert["FT2"]["CM2"] = 1.0/convert["CM2"]["FT2"];
     convert["M2"]["IN2"] = convert["M"]["IN"]*convert["M"]["IN"];
     convert["IN2"]["M2"] = 1.0/convert["M2"]["IN2"];
     convert["FT2"]["IN2"] = 144.0;
@@ -86,6 +87,12 @@ Element::Element(const string& nm)
     convert["FT3"]["M3"] = 1.0/convert["M3"]["FT3"];
     convert["LTR"]["IN3"] = 61.0237441;
     convert["IN3"]["LTR"] = 1.0/convert["LTR"]["IN3"];
+    convert["GAL"]["FT3"] = 0.133681;
+    convert["FT3"]["GAL"] = 1.0/convert["GAL"]["FT3"];
+    convert["IN3"]["GAL"] = convert["IN3"]["FT3"]*convert["FT3"]["GAL"];
+    convert["LTR"]["GAL"] = convert["LTR"]["IN3"]*convert["IN3"]["GAL"];
+    convert["M3"]["GAL"] = 1000.*convert["LTR"]["GAL"];
+    convert["CC"]["GAL"] = convert["CC"]["IN3"]*convert["IN3"]["GAL"];
     // Mass & Weight
     convert["LBS"]["KG"] = 0.45359237;
     convert["KG"]["LBS"] = 1.0/convert["LBS"]["KG"];
@@ -119,6 +126,7 @@ Element::Element(const string& nm)
     convert["KTS"]["FT/SEC"] = 1.68781;
     convert["FT/SEC"]["KTS"] = 1.0/convert["KTS"]["FT/SEC"];
     convert["M/S"]["FT/S"] = 3.2808399;
+    convert["M/S"]["KTS"] = convert["M/S"]["FT/S"]/convert["KTS"]["FT/SEC"];
     convert["M/SEC"]["FT/SEC"] = 3.2808399;
     convert["FT/S"]["M/S"] = 1.0/convert["M/S"]["FT/S"];
     convert["M/SEC"]["FT/SEC"] = 3.2808399;
@@ -148,12 +156,18 @@ Element::Element(const string& nm)
     convert["PA"]["LBS/FT2"] = 1.0/convert["LBS/FT2"]["PA"];
     // Mass flow
     convert["KG/MIN"]["LBS/MIN"] = convert["KG"]["LBS"];
+    convert["KG/SEC"]["LBS/SEC"] = convert["KG"]["LBS"];
+    convert ["N/SEC"]["LBS/SEC"] = 0.224808943;
+    convert ["LBS/SEC"]["N/SEC"] = 1.0/convert ["N/SEC"]["LBS/SEC"];
     // Fuel Consumption
     convert["LBS/HP*HR"]["KG/KW*HR"] = 0.6083;
     convert["KG/KW*HR"]["LBS/HP*HR"] = 1.0/convert["LBS/HP*HR"]["KG/KW*HR"];
     // Density
     convert["KG/L"]["LBS/GAL"] = 8.3454045;
     convert["LBS/GAL"]["KG/L"] = 1.0/convert["KG/L"]["LBS/GAL"];
+    // Gravitational
+    convert["FT3/SEC2"]["M3/SEC2"] = convert["FT3"]["M3"];
+    convert["M3/SEC2"]["FT3/SEC2"] = convert["M3"]["FT3"];
 
     // Length
     convert["M"]["M"] = 1.00;
@@ -169,6 +183,7 @@ Element::Element(const string& nm)
     convert["M3"]["M3"] = 1.0;
     convert["FT3"]["FT3"] = 1.0;
     convert["LTR"]["LTR"] = 1.0;
+    convert["GAL"]["GAL"] = 1.0;
     // Mass & Weight
     convert["KG"]["KG"] = 1.00;
     convert["LBS"]["LBS"] = 1.00;
@@ -219,12 +234,16 @@ Element::Element(const string& nm)
     convert["LBS/SEC"]["LBS/SEC"] = 1.00;
     convert["KG/MIN"]["KG/MIN"] = 1.0;
     convert["LBS/MIN"]["LBS/MIN"] = 1.0;
+    convert["N/SEC"]["N/SEC"] = 1.0;
     // Fuel Consumption
     convert["LBS/HP*HR"]["LBS/HP*HR"] = 1.0;
     convert["KG/KW*HR"]["KG/KW*HR"] = 1.0;
     // Density
     convert["KG/L"]["KG/L"] = 1.0;
     convert["LBS/GAL"]["LBS/GAL"] = 1.0;
+    // Gravitational
+    convert["FT3/SEC2"]["FT3/SEC2"] = 1.0;
+    convert["M3/SEC2"]["M3/SEC2"] = 1.0;
   }
 }
 
@@ -232,22 +251,27 @@ Element::Element(const string& nm)
 
 Element::~Element(void)
 {
-  for (unsigned int i=0; i<children.size(); i++) delete children[i];
-  data_lines.clear();
-  attributes.clear();
-  attribute_key.clear();
+  for (unsigned int i = 0; i < children.size(); ++i)
+    children[i]->SetParent(0);
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 string Element::GetAttributeValue(const string& attr)
 {
-  int select=-1;
-  for (unsigned int i=0; i<attribute_key.size(); i++) {
-    if (attribute_key[i] == attr) select = i;
-  }
-  if (select < 0) return string("");
-  else return attributes[attr];
+  if (HasAttribute(attr))  return attributes[attr];
+  else                     return ("");
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+bool Element::SetAttributeValue(const std::string& key, const std::string& value)
+{
+  bool ret = HasAttribute(key);
+  if (ret)
+    attributes[key] = value;
+
+  return ret;
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -256,8 +280,25 @@ double Element::GetAttributeValueAsNumber(const string& attr)
 {
   string attribute = GetAttributeValue(attr);
 
-  if (attribute.empty()) return HUGE_VAL;
-  else return (atof(attribute.c_str()));
+  if (attribute.empty()) {
+    std::stringstream s;
+    s << ReadFrom() << "Expecting numeric attribute value, but got no data";
+    cerr << s.str() << endl;
+    throw length_error(s.str());
+  }
+  else {
+    double number=0;
+    if (is_number(trim(attribute)))
+      number = atof(attribute.c_str());
+    else {
+      std::stringstream s;
+      s << ReadFrom() << "Expecting numeric attribute value, but got: " << attribute;
+      cerr << s.str() << endl;
+      throw invalid_argument(s.str());
+    }
+
+    return (number);
+  }
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -300,12 +341,33 @@ string Element::GetDataLine(unsigned int i)
 double Element::GetDataAsNumber(void)
 {
   if (data_lines.size() == 1) {
-    return atof(data_lines[0].c_str());
+    double number=0;
+    if (is_number(trim(data_lines[0])))
+      number = atof(data_lines[0].c_str());
+    else {
+      std::stringstream s;
+      s << ReadFrom() << "Expected numeric value, but got: " << data_lines[0];
+      cerr << s.str() << endl;
+      throw invalid_argument(s.str());
+    }
+
+    return number;
   } else if (data_lines.size() == 0) {
-    return HUGE_VAL;
+    std::stringstream s;
+    s << ReadFrom() << "Expected numeric value, but got no data";
+    cerr << s.str() << endl;
+    throw length_error(s.str());
   } else {
-    cerr << "Attempting to get single data value from multiple lines in element " << name << endl;
-    return HUGE_VAL;
+    cerr << ReadFrom() << "Attempting to get single data value in element "
+         << "<" << name << ">" << endl
+         << " from multiple lines:" << endl;
+    for(unsigned int i=0; i<data_lines.size(); ++i)
+      cerr << data_lines[i] << endl;
+    std::stringstream s;
+    s << ReadFrom() << "Attempting to get single data value in element "
+      << "<" << name << ">"
+      << " from multiple lines (" << data_lines.size() << ").";
+    throw length_error(s.str());
   }
 }
 
@@ -368,10 +430,36 @@ double Element::FindElementValueAsNumber(const string& el)
 {
   Element* element = FindElement(el);
   if (element) {
-    return element->GetDataAsNumber();
+    double value = element->GetDataAsNumber();
+    value = DisperseValue(element, value);
+    return value;
   } else {
-    cerr << "Attempting to get single data value from multiple lines" << endl;
-    return 0;
+    std::stringstream s;
+    s << ReadFrom() << "Attempting to get non-existent element " << el;
+    cerr << s.str() << endl;
+    throw length_error(s.str());
+  }
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+bool Element::FindElementValueAsBoolean(const string& el)
+{
+  Element* element = FindElement(el);
+  if (element) {
+    // check value as an ordinary number
+    double value = element->GetDataAsNumber();
+
+    // now check how it should return data
+    if (value == 0) {
+      return false;
+    } else {
+      return true;
+    }
+  } else {
+    cerr << ReadFrom() << "Attempting to get non-existent element " << el << " ;returning false"
+         << endl;
+    return false;
   }
 }
 
@@ -394,30 +482,63 @@ double Element::FindElementValueAsNumberConvertTo(const string& el, const string
   Element* element = FindElement(el);
 
   if (!element) {
-    cerr << "Attempting to get non-existent element " << el << endl;
-    exit(0);
+    std::stringstream s;
+    s << ReadFrom() << "Attempting to get non-existent element " << el;
+    cerr << s.str() << endl;
+    throw length_error(s.str());
   }
 
   string supplied_units = element->GetAttributeValue("unit");
 
   if (!supplied_units.empty()) {
     if (convert.find(supplied_units) == convert.end()) {
-      cerr << endl << "Supplied unit: \"" << supplied_units << "\" does not exist (typo?). Add new unit"
-           << " conversion in FGXMLElement.cpp." << endl;
-      exit(-1);
+      std::stringstream s;
+      s << element->ReadFrom() << "Supplied unit: \"" << supplied_units
+        << "\" does not exist (typo?).";
+      cerr << s.str() << endl;
+      throw invalid_argument(s.str());
     }
     if (convert[supplied_units].find(target_units) == convert[supplied_units].end()) {
-      cerr << endl << "Supplied unit: \"" << supplied_units << "\" cannot be converted to "
-                   << target_units << ". Add new unit conversion in FGXMLElement.cpp or fix typo" << endl;
-      exit(-1);
+      std::stringstream s;
+      s << element->ReadFrom() << "Supplied unit: \"" << supplied_units
+        << "\" cannot be converted to " << target_units;
+      cerr << s.str() << endl;
+      throw invalid_argument(s.str());
     }
   }
 
   double value = element->GetDataAsNumber();
+
+  // Sanity check for angular values
+  if ((supplied_units == "RAD") && (fabs(value) > 2 * M_PI)) {
+    cerr << element->ReadFrom() << element->GetName() << " value "
+         << value << " RAD is outside the range [ -2*M_PI RAD ; +2*M_PI RAD ]"
+         << endl;
+  }
+  if ((supplied_units == "DEG") && (fabs(value) > 360.0)) {
+    cerr << element->ReadFrom() << element->GetName() << " value "
+         << value << " DEG is outside the range [ -360 DEG ; +360 DEG ]"
+         << endl;
+  }
+
+
   if (!supplied_units.empty()) {
     value *= convert[supplied_units][target_units];
   }
 
+  if ((target_units == "RAD") && (fabs(value) > 2 * M_PI)) {
+    cerr << element->ReadFrom() << element->GetName() << " value "
+         << value << " RAD is outside the range [ -2*M_PI RAD ; +2*M_PI RAD ]"
+         << endl;
+  }
+  if ((target_units == "DEG") && (fabs(value) > 360.0)) {
+    cerr << element->ReadFrom() << element->GetName() << " value "
+         << value << " DEG is outside the range [ -360 DEG ; +360 DEG ]"
+         << endl;
+  }
+
+  value = DisperseValue(element, value, supplied_units, target_units);
+
   return value;
 }
 
@@ -430,20 +551,26 @@ double Element::FindElementValueAsNumberConvertFromTo( const string& el,
   Element* element = FindElement(el);
 
   if (!element) {
-    cerr << "Attempting to get non-existent element " << el << endl;
-    exit(0);
+    std::stringstream s;
+    s << ReadFrom() << "Attempting to get non-existent element " << el;
+    cerr << s.str() << endl;
+    throw length_error(s.str());
   }
 
   if (!supplied_units.empty()) {
     if (convert.find(supplied_units) == convert.end()) {
-      cerr << endl << "Supplied unit: \"" << supplied_units << "\" does not exist (typo?). Add new unit"
-           << " conversion in FGXMLElement.cpp." << endl;
-      exit(-1);
+      std::stringstream s;
+      s << element->ReadFrom() << "Supplied unit: \"" << supplied_units
+        << "\" does not exist (typo?).";
+      cerr << s.str() << endl;
+      throw invalid_argument(s.str());
     }
     if (convert[supplied_units].find(target_units) == convert[supplied_units].end()) {
-      cerr << endl << "Supplied unit: \"" << supplied_units << "\" cannot be converted to "
-                   << target_units << ". Add new unit conversion in FGXMLElement.cpp or fix typo" << endl;
-      exit(-1);
+      std::stringstream s;
+      s << element->ReadFrom() << "Supplied unit: \"" << supplied_units
+        << "\" cannot be converted to " << target_units;
+      cerr << s.str() << endl;
+      throw invalid_argument(s.str());
     }
   }
 
@@ -452,6 +579,8 @@ double Element::FindElementValueAsNumberConvertFromTo( const string& el,
     value *= convert[supplied_units][target_units];
   }
 
+  value = DisperseValue(element, value, supplied_units, target_units);
+
   return value;
 }
 
@@ -466,14 +595,18 @@ FGColumnVector3 Element::FindElementTripletConvertTo( const string& target_units
 
   if (!supplied_units.empty()) {
     if (convert.find(supplied_units) == convert.end()) {
-      cerr << endl << "Supplied unit: \"" << supplied_units << "\" does not exist (typo?). Add new unit"
-           << " conversion in FGXMLElement.cpp." << endl;
-      exit(-1);
+      std::stringstream s;
+      s << ReadFrom() << "Supplied unit: \"" << supplied_units
+        << "\" does not exist (typo?).";
+      cerr << s.str() << endl;
+      throw invalid_argument(s.str());
     }
     if (convert[supplied_units].find(target_units) == convert[supplied_units].end()) {
-      cerr << endl << "Supplied unit: \"" << supplied_units << "\" cannot be converted to "
-                   << target_units << ". Add new unit conversion in FGXMLElement.cpp or fix typo" << endl;
-      exit(-1);
+      std::stringstream s;
+      s << ReadFrom() << "Supplied unit: \"" << supplied_units
+        << "\" cannot be converted to " << target_units;
+      cerr << s.str() << endl;
+      throw invalid_argument(s.str());
     }
   }
 
@@ -482,36 +615,84 @@ FGColumnVector3 Element::FindElementTripletConvertTo( const string& target_units
   if (item) {
     value = item->GetDataAsNumber();
     if (!supplied_units.empty()) value *= convert[supplied_units][target_units];
+    triplet(1) = DisperseValue(item, value, supplied_units, target_units);
   } else {
-    value = 0.0;
+    triplet(1) = 0.0;
   }
-  triplet(1) = value;
+
 
   item = FindElement("y");
   if (!item) item = FindElement("pitch");
   if (item) {
     value = item->GetDataAsNumber();
     if (!supplied_units.empty()) value *= convert[supplied_units][target_units];
+    triplet(2) = DisperseValue(item, value, supplied_units, target_units);
   } else {
-    value = 0.0;
+    triplet(2) = 0.0;
   }
-  triplet(2) = value;
 
   item = FindElement("z");
   if (!item) item = FindElement("yaw");
   if (item) {
     value = item->GetDataAsNumber();
     if (!supplied_units.empty()) value *= convert[supplied_units][target_units];
+    triplet(3) = DisperseValue(item, value, supplied_units, target_units);
   } else {
-    value = 0.0;
+    triplet(3) = 0.0;
   }
-  triplet(3) = value;
 
   return triplet;
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
+double Element::DisperseValue(Element *e, double val, const std::string& supplied_units,
+                              const std::string& target_units)
+{
+  double value=val;
+
+  bool disperse = false;
+  try {
+    char* num = getenv("JSBSIM_DISPERSE");
+    if (num) {
+      disperse = (atoi(num) == 1);  // set dispersions
+    }
+  } catch (...) {                   // if error set to false
+    disperse = false;
+    std::cerr << "Could not process JSBSIM_DISPERSE environment variable: Assumed NO dispersions." << endl;
+  }
+
+  if (e->HasAttribute("dispersion") && disperse) {
+    double disp = e->GetAttributeValueAsNumber("dispersion");
+    if (!supplied_units.empty()) disp *= convert[supplied_units][target_units];
+    string attType = e->GetAttributeValue("type");
+    if (attType == "gaussian" || attType == "gaussiansigned") {
+      double grn = FGJSBBase::GaussianRandomNumber();
+    if (attType == "gaussian") {
+      value = val + disp*grn;
+      } else { // Assume gaussiansigned
+        value = (val + disp*grn)*(fabs(grn)/grn);
+      }
+    } else if (attType == "uniform" || attType == "uniformsigned") {
+      double urn = ((((double)rand()/RAND_MAX)-0.5)*2.0);
+      if (attType == "uniform") {
+      value = val + disp * urn;
+      } else { // Assume uniformsigned
+        value = (val + disp * urn)*(fabs(urn)/urn);
+      }
+    } else {
+      std::stringstream s;
+      s << ReadFrom() << "Unknown dispersion type" << attType;
+      cerr << s.str() << endl;
+      throw domain_error(s.str());
+    }
+
+  }
+  return value;
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
 void Element::Print(unsigned int level)
 {
   unsigned int i, spaces;
@@ -519,9 +700,11 @@ void Element::Print(unsigned int level)
   level+=2;
   for (spaces=0; spaces<=level; spaces++) cout << " "; // format output
   cout << "Element Name: " << name;
-  for (i=0; i<attributes.size(); i++) {
-    cout << "  " << attribute_key[i] << " = " << attributes[attribute_key[i]];
-  }
+
+  map<string, string>::iterator it;
+  for (it = attributes.begin(); it != attributes.end(); ++it)
+    cout << "  " << it->first << " = " << it->second;
+
   cout << endl;
   for (i=0; i<data_lines.size(); i++) {
     for (spaces=0; spaces<=level; spaces++) cout << " "; // format output
@@ -536,7 +719,6 @@ void Element::Print(unsigned int level)
 
 void Element::AddAttribute(const string& name, const string& value)
 {
-  attribute_key.push_back(name);
   attributes[name] = value;
 }
 
@@ -553,4 +735,34 @@ void Element::AddData(string d)
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
+string Element::ReadFrom(void) const
+{
+  ostringstream message;
+
+  message << endl
+          << "In file " << GetFileName() << ": line " << GetLineNumber()
+          << endl;
+
+  return message.str();
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+void Element::MergeAttributes(Element* el)
+{
+  map<string, string>::iterator it;
+
+  for (it=el->attributes.begin(); it != el->attributes.end(); ++it) {
+    if (attributes.find(it->first) == attributes.end())
+      attributes[it->first] = it->second;
+    else {
+      if (FGJSBBase::debug_lvl > 0 && (attributes[it->first] != it->second))
+        cout << el->ReadFrom() << " Attribute '" << it->first << "' is overridden in file "
+             << GetFileName() << ": line " << GetLineNumber() << endl
+             << " The value '" << attributes[it->first] << "' will be used instead of '"
+             << it->second << "'." << endl;
+    }
+  }
+}
+
 } // end namespace JSBSim
